; chipmunk wrapper for purebasic by eso
; This assumes the library was compiled using gcc, prepared using eso's wrapper generation tool.
;
; special notes: cpFloat is double everywhere except 32-bit MacOS, where it's single-precision
; consequently, on 32-bit non-MacOS targets, we have padding issues; so only on those
; does the padvalue() macro do anything. On MacOS, or all 64-bit targets, padvalue() evaluates
; to nothing.
;
; needed padding locations determined with gcc using -Wpadded
;
; There's still a few structures that need padding, once they're added to this include:
; struct pNearestPointQueryInfo -> P
; struct cpSegmentQueryInfo -> t
; struct cpContactPointSet -> points
; struct cpDampedSpring -> target_vrn
; struct struct cpDampedRotarySpring -> target_wrn

CompilerSelect #PB_Compiler_Processor
  CompilerCase #PB_Processor_x86
    ; MacOS is special here because its CGPoint structure is floats on x86, doubles on x64
    CompilerIf #PB_Compiler_OS = #PB_OS_MacOS
      Macro cpFloat
        f
      EndMacro
      Macro padvalue(str)
      EndMacro
      ; Everyone else uses doubles, since there's no OS level structures to match for vectors
    CompilerElse
      Macro cpFloat
        d
      EndMacro
      Macro padvalue(str)
        PadValue#str.i
      EndMacro
    CompilerEndIf
    
  CompilerCase #PB_Processor_x64
    ; on x64, everyone uses quads and doubles. Yay consistency.
    Macro cpFloat
      d
    EndMacro
    Macro padvalue(str)
    EndMacro
CompilerEndSelect


; Where possible, I've tried to maintain the original "data type".
; These macros ease that.

Macro cpTimestamp 
  i
EndMacro

Macro cpBool 
  i
EndMacro

Macro cpCollisionType
  i
EndMacro

Macro cpHashValue
  i
EndMacro

Macro cpGroup
  i
EndMacro

Macro cpLayers
  i
EndMacro

Macro cpDataPointer
  i
EndMacro

Macro cpArbiterState
  i
EndMacro


Structure cpVect
  x.cpFloat
  y.cpFloat
EndStructure

; a lot of things frequently need a "zero vector" passed to them. This simplifies that.
Global cpvzero.cpVect

Structure cpBB
  left.cpFloat 
  bottom.cpFloat 
  right.cpFloat 
  top.cpFloat 
EndStructure


Structure cpArray
  num.i
  max.i
  *arr
EndStructure

Structure cpComponentNode
  *root.cpBody 
	*Next.cpBody 
	idleTime.cpFloat 
EndStructure


Structure cpBody
  
	; Function that is called to integrate the body's velocity. (Defaults to cpBodyUpdateVelocity)
	*velocity_func.cpBodyVelocityFunc 
	
	; Function that is called to integrate the body's position. (Defaults to cpBodyUpdatePosition)
	*position_func.cpBodyPositionFunc 
	
	; Mass of the body.
	; Must agree with cpBody.m_inv! Use cpBodySetMass() when changing the mass for this reason.
	m.cpFloat 
	; Mass inverse.
	m_inv.cpFloat 
	
	; Moment of inertia of the body.
	; Must agree with cpBody.i_inv! Use cpBodySetMoment() when changing the moment for this reason.
	i.cpFloat 
	; Moment of inertia inverse.
	i_inv.cpFloat 
	
	; Position of the rigid body's center of gravity.
	position.cpVect 
	; Velocity of the rigid body's center of gravity.
	velocity.cpVect 
	; Force acting on the rigid body's center of gravity.
	force.cpVect 
	
	; Rotation of the body around it's center of gravity in radians.
	; Must agree with cpBody.rot! Use cpBodySetAngle() when changing the angle for this reason.
	angle.cpFloat 
	; Angular velocity of the body around it's center of gravity in radians/second.
	angvel.cpFloat 
	; Torque applied to the body around it's center of gravity.
	torque.cpFloat 
	
	; Cached unit length vector representing the angle of the body.
	; Used for fast rotations using cpvrotate().
	rot.cpVect 
	
	; User definable data pointer.
	; Generally this points to your the game object class so you can access it
	; when given a cpBody reference in a callback.
	*datum.cpDataPointer 
	
	padvalue(v_limit)
	; Maximum velocity allowed when updating the velocity.
	v_limit.cpFloat 
	; Maximum rotational rate (in radians/second) allowed when updating the angular velocity.
	w_limit.cpFloat 
	
	
	; Private members
	v_bias.cpVect 
	w_bias.cpFloat 
	
	*space.cpSpace 
	
	*shapeList.cpShape 
	*arbiterList.cpArbiter 
	*constraintList.cpConstraint 
	
	node.cpComponentNode 
  
EndStructure

Structure cpCollisionHandler
  a.cpCollisionType 
	b.cpCollisionType 
	*begin.cpCollisionBeginFunc 
	*preSolve.cpCollisionPreSolveFunc 
	*postSolve.cpCollisionPostSolveFunc 
	*separate.cpCollisionSeparateFunc 
	*datum;
EndStructure


Structure cpSpace
  ; Number of iterations to use in the impulse solver to solve contacts.
  iterations.i
	
  padvalue(gravity)
	; Gravity to pass to rigid bodies when integrating velocity.
	gravity.cpVect
	
	; Damping rate expressed as the fraction of velocity bodies retain each second.
	; A value of 0.9 would mean that each body's velocity will drop 10% per second.
	; The default value is 1.0, meaning no damping is applied.
	; @note This damping value is different than those of cpDampedSpring and cpDampedRotarySpring.
	damping.cpFloat
	
	; Speed threshold for a body to be considered idle.
	; The default value of 0 means to let the space guess a good threshold based on gravity.
	idleSpeedThreshold.cpFloat 
	
	; Time a group of bodies must remain idle in order to fall asleep.
	; Enabling sleeping also implicitly enables the the contact graph.
	; The default value of INFINITY disables the sleeping algorithm.
	sleepTimeThreshold.cpFloat 
	
	; Amount of encouraged penetration between colliding shapes.
	; Used to reduce oscillating contacts and keep the collision cache warm.
	; Defaults to 0.1. If you have poor simulation quality,
	; increase this number as much as possible without allowing visible amounts of overlap.
	collisionSlop.cpFloat 
	
	; Determines how fast overlapping shapes are pushed apart.
	; Expressed as a fraction of the error remaining after each second.
	; Defaults to pow(1.0 - 0.1, 60.0) meaning that Chipmunk fixes 10% of overlap each frame at 60Hz.
	collisionBias.cpFloat 
	
	; Number of frames that contact information should persist.
	; Defaults to 3. There is probably never a reason to change this value.
	collisionPersistence.cpTimestamp 
	
	; Rebuild the contact graph during each step. Must be enabled to use the cpBodyEachArbiter() function.
	; Disabled by default for a small performance boost. Enabled implicitly when the sleeping feature is enabled.
	enableContactGraph.cpBool 
	
	; User definable data pointer.
	; Generally this points to your game's controller or game state
	; class so you can access it when given a cpSpace reference in a callback.
	*datum.cpDataPointer 
	
	; The designated static body for this space.
	; You can modify this body, or replace it with your own static body.
	; By default it points to a statically allocated cpBody in the cpSpace struct.
	*staticBody.cpBody 
	
	; Members below here are private, and should not be tampered with unnecessarily.
	stamp.cpTimestamp
	
	padvalue(curr_dt)
	curr_dt.cpFloat

	*bodies.cpArray
	*rousedBodies.cpArray 
	*sleepingComponents.cpArray 
	
	*staticShapes.cpSpatialIndex 
	*activeShapes.cpSpatialIndex 
	
	*arbiters.cpArray 
	*contactBuffersHead.cpContactBufferHeader 
	*cachedArbiters.cpHashSet 
	*pooledArbiters.cpArray 
	*constraints.cpArray 
	
	*allocatedBuffers.cpArray 
	locked.i
	
	*collisionHandlers.cpHashSet 
	
	defaultHandler.cpCollisionHandler 
	
	skipPostStep.cpBool 
	padvalue(skippoststep)
	*postStepCallbacks.cpArray 
	
	_staticBody.cpBody 
	
EndStructure

Structure cpShape


  ;private
	*klass.cpShapeClass 
	
	; The rigid body this collision shape is attached to.
	*body.cpBody 

	; The current bounding box of the shape.
	bb.cpBB 
	
	; Sensor flag.
	; Sensor shapes call collision callbacks but don't produce collisions.
	sensor.cpBool 
	
	padvalue(elasticity)
	; Coefficient of restitution. (elasticity)
	elasticity.cpFloat 
	; Coefficient of friction.
	friction.cpFloat 
	; Surface velocity used when solving for friction.
	surface_v.cpVect 

	; User definable data pointer.
	; Generally this points to your the game object class so you can access it
	; when given a cpShape reference in a callback.
	*Datum.cpDataPointer 
	
	; Collision type of this shape used when picking collision handlers.
	collision_type.cpCollisionType 
	; Group of this shape. Shapes in the same group don't collide.
	group.cpGroup 
	; Layer bitmask For this shape. Shapes only collide If the bitwise And of their layers is non-zero.
	layers.cpLayers 
	
	;The following are private, do not tamper
	*space.cpSpace 
	
	*Next.cpShape 
	*prev.cpShape 
	
	hashid.cpHashValue 
  
EndStructure

Structure cpArbiterThread
  ;Links to next and previous arbiters in the contact graph.
  *next.cpArbiter
  *prev.cparbiter
EndStructure


; A colliding pair of shapes.
Structure cpArbiter 
	; Calculated value to use for the elasticity coefficient.
	; Override in a pre-solve collision handler for custom behavior.
	elasticity.cpFloat 
	; Calculated value to use for the friction coefficient.
	; Override in a pre-solve collision handler for custom behavior.
	friction.cpFloat 
	 ; Calculated value to use for applying surface velocities.
	; Override in a pre-solve collision handler for custom behavior.
	surface_vr.cpVect 
	
	; All private below here
	*a.cpShape 
	*b.cpShape 
	*body_a.cpBody 
	*body_b.cpBody 
	
	thread_a.cpArbiterThread 
	thread_b.cpArbiterThread 
	
	numContacts.i
	*contacts.cpContact 
	
	stamp.cpTimestamp 
	*handler.cpCollisionHandler 
	swappedColl.cpBool 
	state.cpArbiterState 
EndStructure

Structure cpSegmentShape Extends cpShape
	
	a.cpVect
	b.cpVect
	N.cpVect
	ta.cpVect
	tb.cpVect
	tn.cpVect
	r.cpFloat
	a_tangent.cpVect
	b_tangent.cpVect
EndStructure


Procedure.s cpvstr(*vector.cpVect)
  ProcedureReturn StrD(*vector\x,3) + ", " + StrD(*vector\y,3)
EndProcedure



; IDE Options = PureBasic 5.00 Beta 3 (Windows - x86)
; CursorPosition = 198
; FirstLine = 180
; Folding = ---
; EnableXP
; CurrentDirectory = C:\Users\void\Dropbox\
; CompileSourceDirectory